package org.avaje.config;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Properties;
import java.util.logging.Logger;

/**
 * Helper used to load the PropertyMap.
 */
final class PropertyMapLoader {

	private static final Logger logger = Logger.getLogger(PropertyMapLoader.class.getName());
	

	/**
	 * Load the file returning the property map.
	 * @param p an existing property map to load into.
	 * @param fileName the name of the properties file to load.
	 */
	public static PropertyMap load(PropertyMap p, String fileName){

		InputStream is = findInputStream(fileName);
		if (is == null){
			logger.severe("properties file "+fileName+" not found");
			return p;
		} else {
			return load(p, is);
		}
	}
	
	/**
	 * Load the InputStream returning the property map.
	 * @param propertyMap an existing property map to load into.
	 * @param in the InputStream of the properties file to load.
	 */
	private static PropertyMap load(PropertyMap propertyMap, InputStream in){
		
		Properties props = new Properties();
		try {
			props.load(in);
			in.close();
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
		
		if (propertyMap == null){
			propertyMap = new PropertyMap();
		}
		
		// put values in initially without any evaluation
		Iterator<Entry<Object, Object>> it = props.entrySet().iterator();
		while (it.hasNext()) {
			Map.Entry<Object, Object> entry = it.next();
			String key = ((String)entry.getKey()).toLowerCase();
			String val = ((String)entry.getValue());
			propertyMap.put(key, val);
		}
		
		propertyMap.evaluateProperties();
		
		String otherProps = propertyMap.remove("load.properties");
		if (otherProps != null){
			otherProps = otherProps.replace("\\", "/");
			InputStream is = findInputStream(otherProps);
			if (is == null){
			  throw new IllegalStateException("load.properties "+otherProps+" not found.");
			}
			logger.fine("loading properties from "+otherProps);
			load(propertyMap, is);
		}

		return propertyMap;
	}

	/**
	 * Find the input stream given the file name.
	 */
	private static InputStream findInputStream(String fileName) {

		if (fileName == null) {
			throw new NullPointerException("fileName is null?");
		}

		try {
			File f = new File(fileName);
			if (f.exists()) {
				logger.fine(fileName+" found in file system");
				return new FileInputStream(f);
			} else {
				InputStream in = findInClassPath(fileName);
				if (in != null){
					logger.fine(fileName+" found in classpath");	
				}
				return in;
			}

		} catch (FileNotFoundException ex) {
			// already made the check so this 
			// should never be thrown
			throw new RuntimeException(ex);
		} 
	}

	private static InputStream findInClassPath(String fileName) {
		return Thread.currentThread().getContextClassLoader().getResourceAsStream(fileName);
	}
	
}
